/** 
 *   Test: parse_configuration
 *
 *   (c) Copyright 2006 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   17/07/2006: Created.
 **/

#define _GNU_SOURCE


#include <stdio.h>
#include <stdint.h>
#include <limits.h>
#include <unistd.h>
#include <stdlib.h>

#include "my_stdio.h"
#include "parse_tools.h"

const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - Verbose mode (incremental).",
#       endif
	"  -p\t\t - Print parsed configuration file to stdout.",
	"  -P flag\t - Print only this flags to stdout (see parse_tools.h)."
        "  -e\t\t - Expand expresions on print TAG.",
        "  -t {TAG}\t - Print out this TAG",
	"  conf_file\t - File to parsed. If it is omited, read from stdin.",
        "  -h\t\t - This help !!!.",
        ""
};


int help(void)
{
	int i=0;

        while((char) *help_text[i] != 0) fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");

	return 0;
}


int main(int argc, char * const argv[])
{
        int op;
	int print_flag = 0;
	int tag_flag = 0;
	int expand_flag = 0;
	char *filename = NULL;
	char *std_in = "/dev/stdin";
	struct configuration *conf = NULL;
	struct configuration *tag = NULL;
	char *tagname = NULL;

	SET_VERBOSE_FLAG(PERR);
        while  ((op = getopt(argc, argv, "vt:pP:e")) != EOF)
		switch( op ){
                case 'v':
			INCREMENT_VERBOSE_FLAG();
                        break;
                case 'e':
			expand_flag = 1;
                        break;
                case 'p':
			print_flag = 1;
			break;
                case 'P':
			print_flag = 1;
			tag_flag = atoi(optarg);
                        break;
                case 't':
			tagname = optarg;
                        break;
                case 'h':
                default:
                        printf("Use %s [-v (incr.)][-p | [-e] -t tag_name|-P flag] [conf_file]\n",
                               argv[0]);
			help();
                        exit(-1);
                }
	filename = argv[optind];
	if (!filename) filename = std_in;
	if (filename) {
		read_conf_file(&conf, filename);

		/* Test: */
		
		if( expand_conf_braket_array(conf) != 0 ) {
			PVERB(PERR,"%s: bracket not match.\n", filename);
		}
		

		if (print_flag)
			write_conf_file_flag(conf, "/dev/stdout", tag_flag);
		else if (tagname) {
			if ((tag = find_conf_tag(conf, tagname)) != NULL) {
				if(!expand_flag)
					printf("\"%s\"\n", tag->data);
				else {
					/* The next two lines expand
                                           all in conf structure: */
					put_env_conf(conf);
					unset_env_conf(conf);
					if(tag->pte_data ) 
						printf("\"%s\"\n", 
						       tag->pte_data);
				}
			}
		}
		free_conf(conf);
	}
	
        return 0;
}




